#include <cmdHandler.h>

#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include <socket.h>

#include <logger.h>
#include <PontCoopScheduler.h>
#include <wizHelper.h>


extern const uint8_t CMD_SOCK;

const uint16_t cmdPort = 2000;

void cmdHandlerEngine(void *handle) {
    static uint8_t receiveBuffer[256];

    static uint8_t state = 0;

    static uint8_t banner[] = \
      "MBGW3\n\r" \
      "Type help for usage help\n\r" \
      "or quit to close the connection.\n\r";
      

    int8_t res = 0;
    uint8_t sockState;
    int32_t resultSend;
    int16_t receivedOctets;
    int32_t resultRecv;

    if (isNetworkAvailable()) {
        switch (state) {
        case 0:
            coloredMsg(LOG_YELLOW, "che, initializing socket");

            res = socket(CMD_SOCK, Sn_MR_TCP, cmdPort, SF_IO_NONBLOCK);
            coloredMsg(LOG_YELLOW, "che, socket returns %d", res);

            if (res == CMD_SOCK) {
                coloredMsg(LOG_YELLOW, "che, socket is initialized");
                state = 1;
            } else {
                state = 255;
            }
            break;

        case 1:
            coloredMsg(LOG_YELLOW, "che, listening");
            
            res = listen(CMD_SOCK);
            coloredMsg(LOG_YELLOW, "che, listen returns %d", res);

            if (res == SOCK_BUSY) {
                coloredMsg(LOG_YELLOW, "che, ok, waiting for established");
                state = 2;
            } else {
                state = 255;
            }
            break;

        case 2:
            coloredMsg(LOG_YELLOW, "che, waiting for established");

            sockState = getSn_SR(CMD_SOCK);
            coloredMsg(LOG_YELLOW, "che, socket state is 0x%02x", sockState);

            if (sockState == SOCK_ESTABLISHED) {
                coloredMsg(LOG_YELLOW, "che, connection is established");
                state = 3;
            }
            break;

        case 3:
            coloredMsg(LOG_YELLOW, "che, send banner");

            resultSend = send(CMD_SOCK, banner, strlen(banner));
            coloredMsg(LOG_YELLOW, "che, sent banner, send returns 0x%02x", resultSend);

            state = 4;
            break;

        case 4:
            // coloredMsg(LOG_YELLOW, "che, now waiting for some input");
            receivedOctets = getSn_RX_RSR(CMD_SOCK);
            // coloredMsg(LOG_YELLOW, "che, getSn_RxMAX returns %d", res16);

            if (receivedOctets > 0) {
                memset(receiveBuffer, 0, sizeof(receiveBuffer));
                resultRecv = recv(CMD_SOCK, receiveBuffer, sizeof(receiveBuffer));
                coloredMsg(LOG_YELLOW, "che, recv returns 0x%02x", resultRecv);
                if (resultRecv > 0) {
                    coloredMsg(LOG_YELLOW, "che, received: %s", receiveBuffer);
                }
            }

            break;

        case 255:
            coloredMsg(LOG_YELLOW, "che, error state, will stop here");
            schDel(cmdHandlerEngine, NULL);
            break;
        }
    } else {
        coloredMsg(LOG_YELLOW, "che, network not yet ready");
    }
}

void cmdHandlerInit() {
    schAdd(cmdHandlerEngine, NULL, 0, 100);
}